using Microsoft.Extensions.Logging;
using SecuritySystemBusinessLogic.MailWorker;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.BusinessLogicsContracts;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StoragesContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Enums;

namespace SecuritySystemBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IClientStorage _clientStorage;
        static readonly object locker = new object();
        private readonly AbstractMailWorker _abstractMailWorker;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker abstractMailWorker, IClientStorage clientStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _abstractMailWorker = abstractMailWorker;
            _clientStorage = clientStorage;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен) return false;

            var inserted = _orderStorage.Insert(model);
            if (inserted == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            model.Id = inserted.Id;

            return ChangeStatus(model, OrderStatus.Принят);
        }

        public bool ChangeStatus(OrderBindingModel model, OrderStatus status)
        {
            CheckModel(model, false);
            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (order == null)
            {
                _logger.LogWarning("Read operation failed");
                return false;
            }
            if (order.Status != status - 1)
            {
                _logger.LogWarning("Status change operation failed");
                throw new InvalidOperationException("Текущий статус заказа не может быть переведен в выбранный");
            }
            if (order.ImplementerId.HasValue)
            {
                model.ImplementerId = order.ImplementerId;
            }
            OrderStatus oldStatus = model.Status;
            model.Status = status;
            if (model.Status == OrderStatus.Выдан)
                model.DateImplement = DateTime.Now;

            var updatedOrder = _orderStorage.Update(model);

            if (updatedOrder == null)
            {
                model.Status = oldStatus;
                _logger.LogWarning("Update operation failed");
                return false;
            }
            var orderClient = _clientStorage.GetElement(new ClientSearchModel { Id = updatedOrder.ClientId });
            if (orderClient != null)
            {
                SendMail(orderClient, updatedOrder);
            }
            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (locker)
            {
                return ChangeStatus(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.SecureId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор secure", nameof(model.SecureId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество secure в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Цена заказа должна быть больше 0", nameof(model.Sum));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество элементов в заказе должно быть больше 0", nameof(model.Count));
            }
            _logger.LogInformation("Order. Sum:{Cost}. Id: {Id}", model.Sum, model.Id);
        }

        public OrderViewModel? ReadElement(OrderSearchModel? model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Order Id:{Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        private void SendMail(ClientViewModel clientView, OrderViewModel orderView)
        {
            if (clientView == null || orderView == null)
            {
                return;
            }
            MailSendInfoBindingModel mailSendInfoBindingModel;

            string subject = $"Заказ #{orderView.Id}";
            string orderInfo = $"Ваш заказ #{orderView.Id} от {orderView.DateCreate} стоимостью {orderView.Sum}";

            if (orderView.Status == OrderStatus.Принят)
            {
                mailSendInfoBindingModel = new MailSendInfoBindingModel
                {
                    MailAddress = clientView.Email,
                    Subject = subject,
                    Text = orderInfo + " был принят"
                };
            }
            else
            {
                mailSendInfoBindingModel = new MailSendInfoBindingModel
                {
                    MailAddress = clientView.Email,
                    Subject = subject,
                    Text = orderInfo + $" поменял статус на {orderView.Status}"
                };
            }
            _abstractMailWorker.MailSendAsync(mailSendInfoBindingModel);
        }
    }
}
